/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.client.internal.ProvisioningClient");

dojo.require("dijit.Dialog");
dojo.require("dojo.cookie");
dojo.require("com.ibm.team.repository.web.ui.internal.LoginWidget");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.client.internal","ProvisioningClientMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.client.internal","ProvisioningClientMessages");

var USERID_COOKIE = "JazzLoginCookie";

com.ibm.team.repository.web.client.internal.ProvisioningClient = {
	
	connect: function(url, userid, password, provisionResponseHandler) {
		var connecturl = "connect?url="+url+"&userid="+userid+"&password="+password;
		this._invokeService(connecturl, provisionResponseHandler, arguments);
	},
	
	disconnect: function(provisionResponseHandler) {
		var url = "disconnect";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	installFeature: function(featureId, version, start, provisionResponseHandler) {
		var url = "installFeature?featureId="+featureId+"&version="+version+"&start="+start;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startFeature: function(featureId, version, provisionResponseHandler) {
		var url = "startFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopFeature: function(featureId, version, provisionResponseHandler) {
		var url = "stopFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startBundle: function(id, provisionResponseHandler) {
		var url = "startBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopBundle: function(id, provisionResponseHandler) {
		var url = "stopBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startAll: function(provisionResponseHandler) {
		var url = "startAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopAll: function(provisionResponseHandler) {
		var url = "stopAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeFeature: function(featureId, version, provisionResponseHandler) {
		var url = "removeFeature?featureId="+featureId+"&version="+version;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeAll: function(provisionResponseHandler) {
		var url = "removeAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getStatus: function(core, installed, available, featureId, version, provisionResponseHandler) {
		var url = "status?core="+core+"&installed="+installed+"&available="+available;
		if (featureId && version) {
			url +="&featureId="+featureId+"&version="+version;
		} 
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	isRepositoryAvailable: function(provisionResponseHandler) {
		var url = "isRepositoryAvailable";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getInstallLog: function(provisionResponseHandler) {
		var url = "status?installLog=true";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getBundleStatus: function(bundleId, provisionResponseHandler) {
		var url = "status?bundleId="+bundleId;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	requestReset: function(provisionResponseHandler) {
		var url = "requestReset";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	reloadLogSettings: function(provisionResponseHandler) {
		var url = "reloadLogSettings";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	_invokeService: function(url, provisionResponseHandler, args) {
		var self = this;
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var bindArgs = {
			url: dojo.baseUrl + "../../admin/cmd/"+url,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs) {
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status,
						statusText: ioArgs.xhr.statusText
					};
					if (ioArgs.xhr.status == 401 && ioArgs.args.url.match(/^.*\/isRepositoryAvailable.*$/)) {
						// Capture the scenario where the user cancels out of a Basic auth prompt and display a helpful message
						// rather than a blank "Loading..." screen
						var errMsg = messages.authRefresh; 
						alert(errMsg);
						errorObj.message = errMsg; 					
					}
					provisionResponseHandler.doErrorCallback(errorObj, args);
				} 
				else if (ioArgs.xhr.getResponseHeader("X-com-ibm-team-repository-web-auth-msg") == "authrequired") {
					self._loginPrompt(this, provisionResponseHandler, args, "get");
				}
				else {
					provisionResponseHandler.doCallback(response, args);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs);
	},
	
	_loginPrompt: function(bindArgs, provisionResponseHandler, args, requestMethod, noQueue, errMessage) {
		var workbenchNode = dojo.byId("workbench");
		var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
		if (dialogWidget == null) {
			var self = this;
			var loginWidget = new com.ibm.team.repository.web.ui.internal.LoginWidget(null, workbenchNode);
			dialogWidget = new dijit.Dialog({id:"com-ibm-team-repository-internal-LoginWidgetDialog",
				execute: dojo.hitch(this, function(dialogArgs) {
					// allow the dialogWidget to be closed
					dialogWidget._alreadyInitialized = true;
					dialogWidget.hide();
					dialogWidget.loginState = "inactive";
					if (dialogArgs.rememberUserID == "on") {
						dojo.cookie(USERID_COOKIE, dialogArgs.userid, { expires: 365 });
					} else {
						dojo.cookie(USERID_COOKIE, "");
					}
					self._login(dialogArgs.userid, dialogArgs.password, bindArgs, provisionResponseHandler, args, requestMethod);
				})
			}, loginWidget.domNode);
			dialogWidget.loginWidget = loginWidget;
			dialogWidget.loginState = "inactive";
			loginWidget.connect(loginWidget.loginButton,"onclick",function(event) {
				dialogWidget.execute(dialogWidget.getValues());
			});
			loginWidget.connect(loginWidget.passwordNode.domNode,"onkeypress",function (event) {
				// If the user hits "Enter" in the password field, submit the data
				if (event.keyCode == dojo.keys.ENTER) {
					dialogWidget.execute(dialogWidget.getValues());
				}
			});
		}
		if (errMessage != null) {
			dialogWidget.loginWidget.showErrorMessage(errMessage);
		}
		else {
			dialogWidget.loginWidget.hideErrorMessage();
		}
		var storedUserId = dojo.cookie(USERID_COOKIE);
		if (storedUserId) {
			dialogWidget.loginWidget.setUserId(storedUserId);
			dialogWidget.loginWidget.rememberUserIdNode.setChecked(true);
			var i = 1;
		} else {
			var i = 0;
		}
		if (dialogWidget.loginState == "inactive") {
			document.title = messages.loginRequired + " - " + messages.jazzTeamServer;
			dialogWidget.show();
			setTimeout(dojo.hitch(dialogWidget, function(){
				dijit.focus(dojo.query('input', this.domNode)[i]); 
			}), 50);
			// prevent the user from closing the dialogWidget without submitting
			dialogWidget._alreadyInitialized = false;
			dialogWidget.loginState = "active";
		}
		if (dialogWidget.queuedRequests == null) {
			dialogWidget.queuedRequests = new Array();
		}
		if (noQueue == null || !noQueue) {
			dialogWidget.queuedRequests.push({
				bindArgs: bindArgs,
				requestMethod: requestMethod,
				responseHandler: provisionResponseHandler,
				args: args
			});
		}
	},

	_login: function(userid, password, bindArgs, provisionResponseHandler, args, requestMethod) {
		var self = this;
	    var bindArgs= {
	        contentType: "application/x-www-form-urlencoded; charset=utf-8",
	        url: "j_security_check",
	        content: {
	            j_username: userid,
	            j_password: password
	        },
			handleAs: "json",
			headers: {
				accept: "text/json"
			},
			handle: dojo.hitch(this, function(response, ioArgs){
				var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
				dialogWidget.loginWidget.passwordNode.domNode.value = "";
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status,
						statusText: ioArgs.xhr.statusText
					};
					provisionResponseHandler.doErrorCallback(errorObj, args);
				}
				else if (ioArgs.xhr.getResponseHeader("X-com-ibm-team-repository-web-auth-msg") == "authfailed") {
					self._loginPrompt(bindArgs, provisionResponseHandler, args, requestMethod, true, messages.loginFailed);
				}
				else {
					var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
					if (dialogWidget.queuedRequests != null) {
						while (dialogWidget.queuedRequests.length > 0) {
							var request = dialogWidget.queuedRequests.pop();
							if (request.requestMethod == "post") {
								xhr = dojo.rawXhrPost(request.bindArgs);
							}
							else {
								xhr = dojo.xhrGet(request.bindArgs);
							}
						}
					}
				}
	        })
	    };
	    dojo.xhrPost(bindArgs);
	}
};

})();